(function() {'use strict';

    var Scanner = function() {
        this.PROTOCOL_DOMAIN = PUtil.checkPage.IsFacebook() ? window.location.protocol + '//' + window.location.host : 'https://www.facebook.com';
        this.responseHeaderScanSingle = FPScanSingleResponse;
        this.responseHeaderScan = FPScanResponse;
        this.retryCount = 3;// retry 3 times to detect if user login.
        
        this.UserInfo = {
            hasUserInfo : false,
            userID : null,
            isUnder18 : null
        }
    };

    function checkIsUnder18(data){
        if (data.indexOf("214189648617074") >= 0) {
            return true;
        }
        return false;
    }
       
    Scanner.prototype.getUserInfo = function(notUnder18, isUnder18, notLogIn, whenNetworkError) {
        var that = this;

        SendWebRequest({
            'type' : 'GET',
            'data' : null,
            'dataType' : 'html',
            'url' : that.PROTOCOL_DOMAIN + "/settings?tab=privacy"
        }, function(data) {
            SendWebRequest({
                'type' : 'GET',
                'data' : null,
                'dataType' : 'html',
                'url' : that.PROTOCOL_DOMAIN + "/settings?tab=applications"
            }, function(app_settings) {
                var html = TMExt_$(app_settings);
                var title_array = html.find('span[class="_3aiv"]');
                var desc_array = html.find('div[class="_4na2"]');
                window.app_settings_title_array = title_array;
                window.app_settings_desc_array = desc_array;
                
                that.UserInfo.hasUserInfo = true;
                var parseData = new FacebookScannerHelper().handleResponse_SettingPage(data);
                if (parseData && parseData.user_id) {
                    that.UserInfo.userID = parseData.user_id;
                    that.UserInfo.isUnder18 = checkIsUnder18(data);
                    if(that.UserInfo.isUnder18){
                        return isUnder18();
                    }else{
                        return notUnder18();
                    }   
                } else {
                    if(that.retryCount > 0)
                    {
                        that.retryCount = that.retryCount - 1;
                        that.getUserInfo(notUnder18, isUnder18, notLogIn, whenNetworkError);
                        return;
                    }
                    that.retryCount = 3;
                    // user not logged in
                    return notLogIn();
                }
            });
        }, whenNetworkError);
    };

    Scanner.prototype.scanSingle = function(id, callback) {
        var that = this;
        var scannerHelper = null;
        
        if(that.UserInfo.hasUserInfo){
            if(!that.UserInfo.isUnder18){
                scannerHelper = new FacebookScannerHelper();
                return doScanSingle();
            }else{
                scannerHelper = new FacebookScannerHelper_Under18();
                return doScanSingle();
            }
        }else{
            that.getUserInfo(function(){
                // not under 18
                scannerHelper = new FacebookScannerHelper();
                return doScanSingle();
            },function(){
                // under 18
                scannerHelper = new FacebookScannerHelper_Under18();
                return doScanSingle();
            },function(){
                // not log in
                var responseData = new APIResponseHelper(that.responseHeaderScanSingle, ReturnCodeEnum.NOT_LOGGED_IN.code, ReturnCodeEnum.NOT_LOGGED_IN.description).getResponse();
                PSDebug.error(responseData);
                return callback(responseData);
            },sendWebRequestFailCallback);
        }
        
        function sendWebRequestFailCallback(data) {
            // network error
            var responseData = new APIResponseHelper(that.responseHeaderScanSingle, ReturnCodeEnum.NETWORK_ERROR.code, ReturnCodeEnum.NETWORK_ERROR.description).getResponse();
            PSDebug.error(responseData);
            return callback(responseData);
        }
        
        function doScanSingle() {
            if (!scannerHelper.isValidID(id)) {

                var responseData = new APIResponseHelper(that.responseHeaderScanSingle, ReturnCodeEnum.INVALID_ID.code, ReturnCodeEnum.INVALID_ID.description, 'wrong ID! : ' + id + ', availableIDS: ' + scannerHelper.getAllvalidID()).getResponse();
                PSDebug.error(responseData);
                return callback(responseData);
            }

            function sendWebRequestOKCallback(data) {
                var parseData = scannerHelper.handleResponse(id, data);
                if(parseData && TMExt_$.trim(parseData["Title"]) == ""){
                    // do not show the setting when we find the title is empty
                    parseData = null;
                }
                // parseData = null;
                if (!parseData) {
                    // web content parse error
                    var responseData = new APIResponseHelper(that.responseHeaderScanSingle, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR.code, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR.description, 'Web content parse error! ID: ' + id + ', webData: ' + data).getResponse();
                    PSDebug.error(responseData);
                    return callback(responseData);
                }

                // return correct web content (white list: we only return these items)
                var parseDataFinal = {
                    ID : parseData['ID'],
                    Risk : parseData['Risk'],
                    Title : parseData['Title'],
                    Desc : parseData['Desc'],
                    PossibleValue : parseData['PossibleValue'],
                    possibleFixValue : parseData['possibleFixValue'],
                    Current : parseData['Current'],
                    Suggestion : parseData['Suggestion'],
                    Category : parseData['Category'],
                    Website : parseData['Website']
                };

                var responseData = new APIResponseHelper(that.responseHeaderScanSingle, ReturnCodeEnum.OK.code, ReturnCodeEnum.OK.description, parseDataFinal).getResponse();
                PSDebug.log(responseData);
                return callback(responseData);
            }
            
            SendWebRequest(scannerHelper.getSendData(id, that.UserInfo.userID), sendWebRequestOKCallback, sendWebRequestFailCallback);
        }
    };

    Scanner.prototype.scan = function(callback) {
        var that = this;
        var scanResult = {
            'TimeLineAndTagging' : {
                'ID' : 3,
                'Items' : []
            },
            'PrivacySetting' : {
                'ID' : 2,
                'Items' : []
            },
            'AppsAndWebsites' : {
                'ID' : 8,
                'Items' : []
            }
        };

        var SCANSINGLE_COUNT = FacebookScanResultsTemplate.length;

        // scan single callback
        var countAlreadyScan = 0;
        var needStopCallback = false;
        var isWebResponsePartlyError = false;
        function scanSingleCallback(data) {
            if (needStopCallback) {
                return;
            }

            if (data[that.responseHeaderScanSingle]['ReturnCode'] == ReturnCodeEnum.OK.code) {
                // get right single scan result, add it to final result now
                addSingleToScanResult(data[that.responseHeaderScanSingle]['Response']);
            } else if (data[that.responseHeaderScanSingle]['ReturnCode'] == ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR.code) {
                isWebResponsePartlyError = true;
            } else {
                var responseData = new APIResponseHelper(that.responseHeaderScan, data[that.responseHeaderScanSingle].ReturnCode, data[that.responseHeaderScanSingle].Description, data[that.responseHeaderScanSingle].Response).getResponse();

                PSDebug.error(responseData);

                needStopCallback = true;
                return callback(responseData);
            }

            function addSingleToScanResult(data) {
                switch (parseInt(data['ID'])) {
                    case 20:
                    case 22:
                    case 23:
                    case 24:
                        scanResult.PrivacySetting.Items.push(data);
                        break;
                    case 31:
                    case 33:
                    case 34:
                    case 35:
                    case 36:
                        scanResult.TimeLineAndTagging.Items.push(data);
                        break;
                    case 81:
                    case 82:
                    case 83:
                        scanResult.AppsAndWebsites.Items.push(data);
                        break;
                }
            }

            countAlreadyScan++;
            if (countAlreadyScan == SCANSINGLE_COUNT) {
                // finish all single scan
                if (!isWebResponsePartlyError) {
                    var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.OK.code, ReturnCodeEnum.OK.description, scanResult).getResponse();
                } else {
                    var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR_COULD_USE_PARTLY.code, ReturnCodeEnum.WEBRESPONSE_PARSE_ERROR_COULD_USE_PARTLY.description, scanResult).getResponse();
                }
                PSDebug.log(responseData);
                return callback(responseData);
            }
        }
        
        function sendWebRequestFailCallback(data) {
            // network error
            var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.NETWORK_ERROR.code, ReturnCodeEnum.NETWORK_ERROR.description).getResponse();
            PSDebug.error(responseData);
            return callback(responseData);
        }
        
        function doScan(){
            // scan every single setting
            for (var i = 0; i < SCANSINGLE_COUNT; i++) {
                that.scanSingle(FacebookScanResultsTemplate[i]['ID'], scanSingleCallback);
            }   
            PSDebug.log("do scan finish");
        }
        
        that.getUserInfo(function(){
            // not under 18
            return doScan();
        },function(){
            // under 18
            return doScan();
        },function(){
            // not log in
            var responseData = new APIResponseHelper(that.responseHeaderScan, ReturnCodeEnum.NOT_LOGGED_IN.code, ReturnCodeEnum.NOT_LOGGED_IN.description).getResponse();
            PSDebug.error(responseData);
            return callback(responseData);
        },sendWebRequestFailCallback);
    };

    Scanner.prototype.Website = "FACEBOOK";

    window.FacebookPScanner = Scanner;

    // for android old interface
    var PSScan = {};
    PSScan.scanSingle = function(id, callback) {
        new Scanner().scanSingle(id, function(data) {
            return callback(data['FPScanSingleResponse']['Response']);
        });
    };

    PSScan.scan = function(callback) {
        new Scanner().scan(function(data) {
            return callback(data['FPScanResponse']['Response']);
        });
    };
    window.PSScan = PSScan;
})();
