(function() {'use strict';

    // ================================================
    // ScannerHelper
    // ================================================
    var ScannerHelper = function() {
        this.PROTOCOL_DOMAIN = PUtil.checkPage.IsLinkedIn() ? window.location.protocol + '//' + window.location.host : 'https://www.linkedin.com';
        this.ScanResultsTemplate = LinkedinScanResultsTemplate;
    };
    ScannerHelper.prototype.logHeaderSendData = '[Linkedin Scan -> send data]';
    ScannerHelper.prototype.logHeaderHandleResponseData = '[Linkedin Scan -> handle response data]';
    
    ScannerHelper.prototype.getAllvalidID = function() {
        var allValidID = [];
        for (var i = 0; i < this.ScanResultsTemplate.length; i++) {
            allValidID.push(this.ScanResultsTemplate[i]['ID']);
        }
        return allValidID;
    };
    ScannerHelper.prototype.isValidID = function(id) {
        var allValidID = this.getAllvalidID();
        for (var i = 0; i < allValidID.length; i++) {
            if (id == allValidID[i]) {
                return true;
            }
        }
        return false;
    };

    ScannerHelper.prototype.getScanResultSampleByID = function(id) {
        for (var i = 0; i < this.ScanResultsTemplate.length; i++) {
            if (this.ScanResultsTemplate[i]['ID'] == id) {
                return PUtil.cloneObj(this.ScanResultsTemplate[i]);
            }
        }
    };
    
    ScannerHelper.prototype.getSessionInfo = function(callback) {
        var sessionInfo = [];
        SendWebRequest({
            'type' : 'GET',
            'data' : null,
            'dataType' : 'html',
            'url' : this.PROTOCOL_DOMAIN + "/settings/security-v2"
        }, function(data){
            // parse "csrfToken" and "sourceAlias"
            try{
                var data_$ = TMExt_$(data);
                
                sessionInfo.push({
                    key: "csrfToken",
                    value : data_$.find('input[name="csrfToken"]').attr('value')
                });
                sessionInfo.push({
                    key: "sourceAlias",
                    value : data_$.find('input[name="sourceAlias"]').attr('value')
                });
            }catch(e){
                PSDebug.error("can not get sessionInfo");
                PSDebug.error(data);
                sessionInfo = null;
            }
            return callback(sessionInfo);
        });
    };

    ScannerHelper.prototype.getSendData = function(id) {
        var id_url = {
            "11" : "/settings/activity-broadcasts",
            "12" : "/settings/activity-visibility",
            "13" : "/settings/data-sharing",
            "14" : "/settings/enhanced-advertising",
            "16" : "/settings/security-v2"
        };
        
        var params = {
            'type' : 'GET',
            'data' : null,
            'dataType' : 'html',
            'url' : this.PROTOCOL_DOMAIN + id_url[id]
        };

        new SendDataLogger(this.logHeaderSendData + ' : ' + id, params).getLog();
        return params;
    };
    
    ScannerHelper.prototype.handleResponse = function(id, data) {
        new ResponseHandlerLogger(this.logHeaderHandleResponseData + ', ID : ' + id, data).getLog();
        var scanResult = this.getScanResultSampleByID(id);
        // parse html page to render "Title", "Desc", "Current" 
        switch (String(id)) {
            case '11':
                try {
                    var data_$ = TMExt_$(data);
                    
                    var title = data_$.find("label[for='activity-activity-editActivityBroadcasts']").text();
                    scanResult.Title = TMExt_$.trim(title);
                    scanResult.Desc = data_$.find("p.note").text();
                    scanResult.Current = data_$.find("#activity-activity-editActivityBroadcasts")[0].checked ? 1 : 0;
                    return scanResult;
                } catch (e) {
                    PSDebug.error(e);
                    return null;
                }
                break;
            case '12':
                try {
                    var data_$ = TMExt_$(data);
                    
                    scanResult.Title = data_$.find("div.demodalized h1:eq(0)").text();
                    scanResult.Desc = data_$.find("form[name='editActivityFeed'] p:eq(0)").text();
                    
                    scanResult.PossibleValue = {};
                    var options = data_$.find("#activityFeed-editActivityFeed option");
                    var count = 0;
                    for (var i=0; i < options.length; i++) {
                        var current_option_$ = options.eq(i);
                        scanResult.PossibleValue[count] = current_option_$.text();
                        // check if selected
                        if(current_option_$.attr('selected') !== undefined){
                            scanResult.Current = i;
                        } 
                        count += 1;
                    };

                    return scanResult;
                } catch (e) {
                    PSDebug.error(e);
                    return null;
                }
                break;
            case '13':
                try {
                    var data_$ = TMExt_$(data);
                    
                    scanResult.Title = data_$.find("div.demodalized h1:eq(0)").text();
                    scanResult.Desc = scanResult.Title;
                    scanResult.Current = data_$.find("#dataSharingParam-dataSharingParam-dataSharing")[0].checked ? 1 : 0;
                    return scanResult;
                } catch (e) {
                    PSDebug.error(e);
                    return null;
                }
                break;
            case '14':
                try {
                    var data_$ = TMExt_$(data);
                    
                    scanResult.Title = data_$.find('div.demodalized h1:eq(0)').text();
                    scanResult.Desc = data_$.find("label[for='enhancedAdvertisingParam-enhancedAdvertisingParam-editReceivingMarketing']").text() + data_$.find("label[for='affiliateAdvertisingParam-affiliateAdvertisingParam-editReceivingMarketing']").text(); 
                    
                    // if any ad setting is on, current is on. Otherwise, current is off.
                    var third_party_web = data_$.find("#enhancedAdvertisingParam-enhancedAdvertisingParam-editReceivingMarketing")[0].checked ? 1 : 0;
                    var third_party_data = data_$.find("#affiliateAdvertisingParam-affiliateAdvertisingParam-editReceivingMarketing")[0].checked ? 1 : 0;
                    scanResult.Current = (third_party_web || third_party_data) ? 1 : 0; 
                    return scanResult;
                } catch (e) {
                    PSDebug.error(e);
                    return null;
                }
                break;
            case '16':
                try {
                    var data_$ = TMExt_$(data);
                    
                    var title = data_$.find("label[for='fullSsl-fullSsl-editSecurity']").contents().filter(function(){return this.nodeType === 3}).eq(0).text();
                    scanResult.Title = TMExt_$.trim(title);
                    scanResult.Desc = scanResult.Title;
                    scanResult.Current = data_$.find("#fullSsl-fullSsl-editSecurity")[0].checked ? 1 : 0;
                    return scanResult;
                } catch (e) {
                    PSDebug.error(e);
                    return null;
                }
                break;
        }
        
        return null;
    };
    
    window.LinkedinScannerHelper = ScannerHelper;
})();