(function() {'use strict';
    
    /* Util */
    var timeOutCount = 0;
    function isPageLoadComplete(data) {
        timeOutCount += TWITTER_TIME_INTERVAL_CHECK_LOAD_COMPLETE;
        if (timeOutCount >= TWITTER_TIMEOUT_LOAD_COMPLETE) {
            return true;
        }

        // check load complete by verifing if save button is visible
        if (data.find('#settings_save').is(':visible')) {
            return true;
        }
        return false;
    }
    
    window.TwitterIsPageLoadComplete = isPageLoadComplete;
    
    var ScannerHelper = TwitterScannerHelper;
    var FixHelper = TwitterFixHelper;

    var Fixer = function() {
        this.ScannerHelper = new ScannerHelper();
        this.FixHelper = new FixHelper();
        this.responseHeaderFixSingle = TPFixSingleResponse;
        this.responseHeaderClickSave = TPClickSaveResponse;
    };

    Fixer.prototype.fixSingle = function(id, value, callback) {
        if (id == 'SAVE') {
            this.clickSave(callback);
            return;
        }
        var that = this;
        // params->id invalid error
        if (!that.ScannerHelper.isValidID(id)) {
            var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.INVALID_ID.code, ReturnCodeEnum.INVALID_ID.description, 'wrong ID! : ' + id + ', availableIDS: ' + that.ScannerHelper.getAllvalidID()).getResponse();
            PSDebug.error(responseData);
            return callback(responseData);
        }

        // params->value invalid error
        if (!that.FixHelper.isValidFixValue(id, value)) {
            var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.INVALID_FIX_VALUE.code, ReturnCodeEnum.INVALID_FIX_VALUE.description, 'wrong fix value! : ' + value + ', availableValues: ' + that.FixHelper.getValidFixValueByID(id)).getResponse();
            PSDebug.error(responseData);
            return callback(responseData);
        }

        function performFixFinish(data) {
            var parseData = that.FixHelper.handleResponse(id, value, data);
            if (!parseData) {
                // web content parse error
                var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.PERFORM_CLICK_FAIL.code, ReturnCodeEnum.PERFORM_CLICK_FAIL.description, 'perform click fail! ID: ' + id + ', value: ' + value + ', result: ' + data).getResponse();
                PSDebug.error(responseData);
                return callback(responseData);
            }

            var responseData = new APIResponseHelper(that.responseHeaderFixSingle, ReturnCodeEnum.OK.code, ReturnCodeEnum.OK.description, parseData).getResponse();
            PSDebug.log(responseData);
            return callback(responseData);
        }

        function triggerDoFix() {
            // wait for page load complete
            if (TwitterIsPageLoadComplete(TMExt_$(document))) {
                setTimeout(doFix, TWITTER_WAITING_BEFORE_REAL_SCAN);
            } else {
                setTimeout(triggerDoFix, TWITTER_TIME_INTERVAL_CHECK_LOAD_COMPLETE);
            }
            function doFix(){
                // return true if we could find the dom and perform click, otherwise, return false
                var fixResult = that.FixHelper.performFix(id, value);
                performFixFinish(fixResult);    
            }
        }

        triggerDoFix();
    };

    Fixer.prototype.clickSave = function(callback) {
        var that = this;
        var save_result = that.FixHelper.performSave();

        performClickSaveFinish(save_result);

        function performClickSaveFinish(data) {
            var parseData = that.FixHelper.handleResponse('save_button', null, data);
            if (!parseData) {
                var responseData = new APIResponseHelper(that.responseHeaderClickSave, ReturnCodeEnum.PERFORM_CLICK_FAIL.code, ReturnCodeEnum.PERFORM_CLICK_FAIL.description, 'perform click fail! ID: ' + 'save_button' + ', value: ' + 'null' + ', result: ' + 'false').getResponse();
                PSDebug.error(responseData);
                return callback(responseData);
            }

            var responseData = new APIResponseHelper(that.responseHeaderClickSave, ReturnCodeEnum.OK.code, ReturnCodeEnum.OK.description, parseData).getResponse();
            PSDebug.log(responseData);
            return callback(responseData);
        }
    };

    window.TwitterPFixer = Fixer;
})();
